
use std::ffi::CString;
//use std::fmt;

#[link(name="logsplrs_x64", kind="static")]

extern {
    fn logInit(dir: *const u8, prefix: *const u8, wf: i32, clopt: i32, llv: i32, lsw: i32, bom: i32 )->bool;
    fn logInitProfile(dir: *const u8)->bool;
    fn logTerm();
    fn _log(data: *const u8);
    fn logDebug(data: *const u8);
    fn logInfo(data: *const u8);
    fn logWarning(data: *const u8);
    fn logError(data: *const u8);
    fn logCritical(data: *const u8);
    fn w32ResetErrCode();
    fn w32ErrLog(file_fnc: *const u8, line: i32);
    fn getLastErr()->u32;
    fn logDumpUTF8(data: *const u8, size: i32, prefix: *const u8);
    fn logDumpRaw(data: *const u8, size: i32, prefix: *const u8);
    fn logPut(data: *const u8, cnt: i32, header: bool, new_line_cnt: i32);
    fn logPutHex(data: *const u8, size: i32, header: bool, new_line_cnt: i32);
    fn makeBitStringUINT32(x: u32, prefix: *const u8);
    fn makeBitStringUSHORT16(x: u16, prefix: *const u8);
    fn makeBitStringUnicodeScalarValue(scalar: u32, prefix: *const u8);
    fn msgBox(msg: *const u8, ttl: *const u8);
    fn msgBoxOkCancel(msg: *const u8, ttl: *const u8)->i32;
    fn profileTimeStart(prefix: *const u8);
    fn profileTimeStop();

}

#[allow(dead_code)]
pub fn log_init(dir: &str, prefix: &str, wf: i32, crlfopt: i32, llv: i32, lsw: i32, bom: i32)->bool{
    unsafe{
        let c_dir= CString::new(dir).unwrap();
        let c_prefix = CString::new(prefix).unwrap();
        logInit(c_dir.as_ptr() as *const u8, 
                c_prefix.as_ptr() as *const u8, wf, crlfopt, llv, lsw, bom)
    }
}

#[allow(dead_code)]
pub fn log_init_profile(dir: &str)->bool{
    unsafe{
        let c_dir= CString::new(dir).unwrap();
        logInitProfile(c_dir.as_ptr() as *const u8)
    }
}

#[allow(dead_code)]
pub fn log_term(){
    unsafe{
        logTerm();
    }
}

#[allow(dead_code)]
pub fn log(data: &str){
    unsafe{
        let c_str = CString::new(data).unwrap();
        _log(c_str.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn log_debug(data: &str){
    unsafe{
        let c_str = CString::new(data).unwrap();
        logDebug(c_str.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn log_info(data: &str){
    unsafe{
        let c_str = CString::new(data).unwrap();
        logInfo(c_str.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn log_warning(data: &str){
    unsafe{
        let c_str = CString::new(data).unwrap();
        logWarning(c_str.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn log_error(data: &str){
    unsafe{
        let c_str = CString::new(data).unwrap();
        logError(c_str.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn log_critical(data: &str){
    unsafe{
        let c_str = CString::new(data).unwrap();
        logCritical(c_str.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn w32_reset_err_code(){
    unsafe{
        w32ResetErrCode();
    }
}

#[allow(dead_code)]
pub fn w32_err_log(finle_fnc: &str, line: i32){
    unsafe{
        let c_str = CString::new(finle_fnc).unwrap();
        w32ErrLog(c_str.as_ptr() as *const u8, line);
    }
}

#[allow(dead_code)]
pub fn get_last_error()->u32{
    unsafe{
        getLastErr()
    }
}

#[allow(dead_code)]
pub fn log_dump_utf8(data: &str, size: i32, prefix: &str){
    unsafe{
        let c_data = CString::new(data).unwrap();
        let c_prefix = CString::new(prefix).unwrap();
        logDumpUTF8(c_data.as_ptr() as *const u8, size, c_prefix.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn log_dump_raw(data: &[u8], size: i32, prefix: &str){
    unsafe{
        //let c_data = CString::new(data).unwrap();
        let c_prefix = CString::new(prefix).unwrap();
        logDumpRaw(data.as_ptr() as *const u8, size, c_prefix.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn log_put(data: &str, len: i32, header: bool, new_line_cnt: i32){
    unsafe{
        let c_data = CString::new(data).unwrap();
        logPut(c_data.as_ptr() as *const u8, len, header, new_line_cnt);
    }
}

#[allow(dead_code)]
pub fn log_put_hex(data: &str, len: i32, header: bool, new_line_cnt: i32){
    unsafe{
        let c_data = CString::new(data).unwrap();
        logPutHex(c_data.as_ptr() as *const u8, len, header, new_line_cnt);
    }
}

#[allow(dead_code)]
pub fn makebit_string_u32(x: u32, prefix: &str){
    unsafe{
        let c_prefix = CString::new(prefix).unwrap();
        makeBitStringUINT32(x, c_prefix.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn makebit_string_us16(x: u16, prefix: &str){
    unsafe{
        let c_prefix = CString::new(prefix).unwrap();
        makeBitStringUSHORT16(x, c_prefix.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn makebit_string_unicode_scalar_value(scalar: u32, prefix: &str)
{
    unsafe{
        let c_prefix = CString::new(prefix).unwrap();
        makeBitStringUnicodeScalarValue(scalar, c_prefix.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn msg_box(msg: &str, ttl: &str){
    unsafe{
        let c_msg = CString::new(msg).unwrap();
        let c_ttl = CString::new(ttl).unwrap();
        msgBox(c_msg.as_ptr() as *const u8, c_ttl.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn msg_box_okcancel(msg: &str, ttl: &str)->i32{
    unsafe{
        let c_msg = CString::new(msg).unwrap();
        let c_ttl = CString::new(ttl).unwrap();
        msgBoxOkCancel(c_msg.as_ptr() as *const u8, c_ttl.as_ptr() as *const u8)
    }
}

#[allow(dead_code)]
pub fn profile_time_start(prefix: &str){
    unsafe{
        let c_prefix = CString::new(prefix).unwrap();
        profileTimeStart(c_prefix.as_ptr() as *const u8);
    }
}

#[allow(dead_code)]
pub fn profile_time_stop(){
    unsafe{
        profileTimeStop();
    }
}

